from typing import List, Any, TypeVar, Callable, Type, cast


T = TypeVar("T")


def from_list(f: Callable[[Any], T], x: Any) -> List[T]:
    assert isinstance(x, list)
    return [f(y) for y in x]


def from_str(x: Any) -> str:
    assert isinstance(x, str)
    return x


def to_class(c: Type[T], x: Any) -> dict:
    assert isinstance(x, c)
    return cast(Any, x).to_dict()


class Technology:
    aliases: List[str]
    autodetect: List[str]
    by: str
    description: str
    files: List[str]
    learn_more_at: str
    name: str
    slug: str

    def __init__(self, aliases: List[str], autodetect: List[str], by: str, description: str, files: List[str], learn_more_at: str, name: str, slug: str) -> None:
        self.aliases = aliases
        self.autodetect = autodetect
        self.by = by
        self.description = description
        self.files = files
        self.learn_more_at = learn_more_at
        self.name = name
        self.slug = slug

    @staticmethod
    def from_dict(obj: Any) -> 'Technology':
        assert isinstance(obj, dict)
        aliases = from_list(from_str, obj.get("aliases"))
        autodetect = from_list(from_str, obj.get("autodetect"))
        by = from_str(obj.get("by"))
        description = from_str(obj.get("description"))
        files = from_list(from_str, obj.get("files"))
        learn_more_at = from_str(obj.get("learn more at"))
        name = from_str(obj.get("name"))
        slug = from_str(obj.get("slug"))
        return Technology(aliases, autodetect, by, description, files, learn_more_at, name, slug)

    def to_dict(self) -> dict:
        result: dict = {}
        result["aliases"] = from_list(from_str, self.aliases)
        result["autodetect"] = from_list(from_str, self.autodetect)
        result["by"] = from_str(self.by)
        result["description"] = from_str(self.description)
        result["files"] = from_list(from_str, self.files)
        result["learn more at"] = from_str(self.learn_more_at)
        result["name"] = from_str(self.name)
        result["slug"] = from_str(self.slug)
        return result


def technologies_from_dict(s: Any) -> List[Technology]:
    return from_list(Technology.from_dict, s)


def technologies_to_dict(x: List[Technology]) -> Any:
    return from_list(lambda x: to_class(Technology, x), x)



from typing import List, Any, TypeVar, Callable, Type, cast


T = TypeVar("T")


def from_list(f: Callable[[Any], T], x: Any) -> List[T]:
    assert isinstance(x, list)
    return [f(y) for y in x]


def from_str(x: Any) -> str:
    assert isinstance(x, str)
    return x


def to_class(c: Type[T], x: Any) -> dict:
    assert isinstance(x, c)
    return cast(Any, x).to_dict()


class Detect:
    files: List[str]
    made_with: List[str]
    search: List[str]

    def __init__(self, files: List[str], made_with: List[str], search: List[str]) -> None:
        self.files = files
        self.made_with = made_with
        self.search = search

    @staticmethod
    def from_dict(obj: Any) -> 'Detect':
        assert isinstance(obj, dict)
        files = from_list(from_str, obj.get("files"))
        made_with = from_list(from_str, obj.get("made with"))
        search = from_list(from_str, obj.get("search"))
        return Detect(files, made_with, search)

    def to_dict(self) -> dict:
        result: dict = {}
        result["files"] = from_list(from_str, self.files)
        result["made with"] = from_list(from_str, self.made_with)
        result["search"] = from_list(from_str, self.search)
        return result


class Tag:
    aliases: List[str]
    description: str
    detect: Detect
    learn_more_at: str
    plural: str
    singular: str

    def __init__(self, aliases: List[str], description: str, detect: Detect, learn_more_at: str, plural: str, singular: str) -> None:
        self.aliases = aliases
        self.description = description
        self.detect = detect
        self.learn_more_at = learn_more_at
        self.plural = plural
        self.singular = singular

    @staticmethod
    def from_dict(obj: Any) -> 'Tag':
        assert isinstance(obj, dict)
        aliases = from_list(from_str, obj.get("aliases"))
        description = from_str(obj.get("description"))
        detect = Detect.from_dict(obj.get("detect"))
        learn_more_at = from_str(obj.get("learn more at"))
        plural = from_str(obj.get("plural"))
        singular = from_str(obj.get("singular"))
        return Tag(aliases, description, detect, learn_more_at, plural, singular)

    def to_dict(self) -> dict:
        result: dict = {}
        result["aliases"] = from_list(from_str, self.aliases)
        result["description"] = from_str(self.description)
        result["detect"] = to_class(Detect, self.detect)
        result["learn more at"] = from_str(self.learn_more_at)
        result["plural"] = from_str(self.plural)
        result["singular"] = from_str(self.singular)
        return result


def tags_from_dict(s: Any) -> List[Tag]:
    return from_list(Tag.from_dict, s)


def tags_to_dict(x: List[Tag]) -> Any:
    return from_list(lambda x: to_class(Tag, x), x)



from ortfodb.configuration import Configuration, configuration_from_dict
from ortfodb.database import Database, database_from_dict
from ortfodb.tags import Tag, tags_from_dict
from ortfodb.technologies import Technology, technologies_from_dict

__all__ = [
    "Configuration",
    "configuration_from_dict",
    "Database",
    "database_from_dict",
    "Tag",
    "tags_from_dict",
    "Technology",
    "technologies_from_dict",
]



from typing import Any, List, Dict, TypeVar, Type, cast, Callable


T = TypeVar("T")


def from_bool(x: Any) -> bool:
    assert isinstance(x, bool)
    return x


def from_str(x: Any) -> str:
    assert isinstance(x, str)
    return x


def from_float(x: Any) -> float:
    assert isinstance(x, (float, int)) and not isinstance(x, bool)
    return float(x)


def from_int(x: Any) -> int:
    assert isinstance(x, int) and not isinstance(x, bool)
    return x


def to_float(x: Any) -> float:
    assert isinstance(x, float)
    return x


def to_class(c: Type[T], x: Any) -> dict:
    assert isinstance(x, c)
    return cast(Any, x).to_dict()


def from_list(f: Callable[[Any], T], x: Any) -> List[T]:
    assert isinstance(x, list)
    return [f(y) for y in x]


def from_dict(f: Callable[[Any], T], x: Any) -> Dict[str, T]:
    assert isinstance(x, dict)
    return { k: f(v) for (k, v) in x.items() }


class Attributes:
    autoplay: bool
    controls: bool
    loop: bool
    muted: bool
    playsinline: bool

    def __init__(self, autoplay: bool, controls: bool, loop: bool, muted: bool, playsinline: bool) -> None:
        self.autoplay = autoplay
        self.controls = controls
        self.loop = loop
        self.muted = muted
        self.playsinline = playsinline

    @staticmethod
    def from_dict(obj: Any) -> 'Attributes':
        assert isinstance(obj, dict)
        autoplay = from_bool(obj.get("autoplay"))
        controls = from_bool(obj.get("controls"))
        loop = from_bool(obj.get("loop"))
        muted = from_bool(obj.get("muted"))
        playsinline = from_bool(obj.get("playsinline"))
        return Attributes(autoplay, controls, loop, muted, playsinline)

    def to_dict(self) -> dict:
        result: dict = {}
        result["autoplay"] = from_bool(self.autoplay)
        result["controls"] = from_bool(self.controls)
        result["loop"] = from_bool(self.loop)
        result["muted"] = from_bool(self.muted)
        result["playsinline"] = from_bool(self.playsinline)
        return result


class Colors:
    primary: str
    secondary: str
    tertiary: str

    def __init__(self, primary: str, secondary: str, tertiary: str) -> None:
        self.primary = primary
        self.secondary = secondary
        self.tertiary = tertiary

    @staticmethod
    def from_dict(obj: Any) -> 'Colors':
        assert isinstance(obj, dict)
        primary = from_str(obj.get("primary"))
        secondary = from_str(obj.get("secondary"))
        tertiary = from_str(obj.get("tertiary"))
        return Colors(primary, secondary, tertiary)

    def to_dict(self) -> dict:
        result: dict = {}
        result["primary"] = from_str(self.primary)
        result["secondary"] = from_str(self.secondary)
        result["tertiary"] = from_str(self.tertiary)
        return result


class Dimensions:
    aspect_ratio: float
    height: int
    width: int

    def __init__(self, aspect_ratio: float, height: int, width: int) -> None:
        self.aspect_ratio = aspect_ratio
        self.height = height
        self.width = width

    @staticmethod
    def from_dict(obj: Any) -> 'Dimensions':
        assert isinstance(obj, dict)
        aspect_ratio = from_float(obj.get("aspectRatio"))
        height = from_int(obj.get("height"))
        width = from_int(obj.get("width"))
        return Dimensions(aspect_ratio, height, width)

    def to_dict(self) -> dict:
        result: dict = {}
        result["aspectRatio"] = to_float(self.aspect_ratio)
        result["height"] = from_int(self.height)
        result["width"] = from_int(self.width)
        return result


class Thumbnails:
    pass

    def __init__(self, ) -> None:
        pass

    @staticmethod
    def from_dict(obj: Any) -> 'Thumbnails':
        assert isinstance(obj, dict)
        return Thumbnails()

    def to_dict(self) -> dict:
        result: dict = {}
        return result


class BlockElement:
    alt: str
    analyzed: bool
    anchor: str
    attributes: Attributes
    caption: str
    colors: Colors
    content: str
    content_type: str
    dimensions: Dimensions
    dist_source: str
    duration: float
    has_sound: bool
    id: str
    index: int
    online: bool
    relative_source: str
    size: int
    text: str
    thumbnails: Thumbnails
    thumbnails_built_at: str
    title: str
    type: str
    url: str

    def __init__(self, alt: str, analyzed: bool, anchor: str, attributes: Attributes, caption: str, colors: Colors, content: str, content_type: str, dimensions: Dimensions, dist_source: str, duration: float, has_sound: bool, id: str, index: int, online: bool, relative_source: str, size: int, text: str, thumbnails: Thumbnails, thumbnails_built_at: str, title: str, type: str, url: str) -> None:
        self.alt = alt
        self.analyzed = analyzed
        self.anchor = anchor
        self.attributes = attributes
        self.caption = caption
        self.colors = colors
        self.content = content
        self.content_type = content_type
        self.dimensions = dimensions
        self.dist_source = dist_source
        self.duration = duration
        self.has_sound = has_sound
        self.id = id
        self.index = index
        self.online = online
        self.relative_source = relative_source
        self.size = size
        self.text = text
        self.thumbnails = thumbnails
        self.thumbnails_built_at = thumbnails_built_at
        self.title = title
        self.type = type
        self.url = url

    @staticmethod
    def from_dict(obj: Any) -> 'BlockElement':
        assert isinstance(obj, dict)
        alt = from_str(obj.get("alt"))
        analyzed = from_bool(obj.get("analyzed"))
        anchor = from_str(obj.get("anchor"))
        attributes = Attributes.from_dict(obj.get("attributes"))
        caption = from_str(obj.get("caption"))
        colors = Colors.from_dict(obj.get("colors"))
        content = from_str(obj.get("content"))
        content_type = from_str(obj.get("contentType"))
        dimensions = Dimensions.from_dict(obj.get("dimensions"))
        dist_source = from_str(obj.get("distSource"))
        duration = from_float(obj.get("duration"))
        has_sound = from_bool(obj.get("hasSound"))
        id = from_str(obj.get("id"))
        index = from_int(obj.get("index"))
        online = from_bool(obj.get("online"))
        relative_source = from_str(obj.get("relativeSource"))
        size = from_int(obj.get("size"))
        text = from_str(obj.get("text"))
        thumbnails = Thumbnails.from_dict(obj.get("thumbnails"))
        thumbnails_built_at = from_str(obj.get("thumbnailsBuiltAt"))
        title = from_str(obj.get("title"))
        type = from_str(obj.get("type"))
        url = from_str(obj.get("url"))
        return BlockElement(alt, analyzed, anchor, attributes, caption, colors, content, content_type, dimensions, dist_source, duration, has_sound, id, index, online, relative_source, size, text, thumbnails, thumbnails_built_at, title, type, url)

    def to_dict(self) -> dict:
        result: dict = {}
        result["alt"] = from_str(self.alt)
        result["analyzed"] = from_bool(self.analyzed)
        result["anchor"] = from_str(self.anchor)
        result["attributes"] = to_class(Attributes, self.attributes)
        result["caption"] = from_str(self.caption)
        result["colors"] = to_class(Colors, self.colors)
        result["content"] = from_str(self.content)
        result["contentType"] = from_str(self.content_type)
        result["dimensions"] = to_class(Dimensions, self.dimensions)
        result["distSource"] = from_str(self.dist_source)
        result["duration"] = to_float(self.duration)
        result["hasSound"] = from_bool(self.has_sound)
        result["id"] = from_str(self.id)
        result["index"] = from_int(self.index)
        result["online"] = from_bool(self.online)
        result["relativeSource"] = from_str(self.relative_source)
        result["size"] = from_int(self.size)
        result["text"] = from_str(self.text)
        result["thumbnails"] = to_class(Thumbnails, self.thumbnails)
        result["thumbnailsBuiltAt"] = from_str(self.thumbnails_built_at)
        result["title"] = from_str(self.title)
        result["type"] = from_str(self.type)
        result["url"] = from_str(self.url)
        return result


class ContentValue:
    blocks: List[BlockElement]
    footnotes: Dict[str, str]
    layout: List[List[str]]
    title: str

    def __init__(self, blocks: List[BlockElement], footnotes: Dict[str, str], layout: List[List[str]], title: str) -> None:
        self.blocks = blocks
        self.footnotes = footnotes
        self.layout = layout
        self.title = title

    @staticmethod
    def from_dict(obj: Any) -> 'ContentValue':
        assert isinstance(obj, dict)
        blocks = from_list(BlockElement.from_dict, obj.get("blocks"))
        footnotes = from_dict(from_str, obj.get("footnotes"))
        layout = from_list(lambda x: from_list(from_str, x), obj.get("layout"))
        title = from_str(obj.get("title"))
        return ContentValue(blocks, footnotes, layout, title)

    def to_dict(self) -> dict:
        result: dict = {}
        result["blocks"] = from_list(lambda x: to_class(BlockElement, x), self.blocks)
        result["footnotes"] = from_dict(from_str, self.footnotes)
        result["layout"] = from_list(lambda x: from_list(from_str, x), self.layout)
        result["title"] = from_str(self.title)
        return result


class DatabaseMetadataClass:
    partial: bool

    def __init__(self, partial: bool) -> None:
        self.partial = partial

    @staticmethod
    def from_dict(obj: Any) -> 'DatabaseMetadataClass':
        assert isinstance(obj, dict)
        partial = from_bool(obj.get("Partial"))
        return DatabaseMetadataClass(partial)

    def to_dict(self) -> dict:
        result: dict = {}
        result["Partial"] = from_bool(self.partial)
        return result


class Metadata:
    additional_metadata: Dict[str, Any]
    aliases: List[str]
    colors: Colors
    database_metadata: DatabaseMetadataClass
    finished: str
    made_with: List[str]
    page_background: str
    private: bool
    started: str
    tags: List[str]
    thumbnail: str
    title_style: str
    wip: bool

    def __init__(self, additional_metadata: Dict[str, Any], aliases: List[str], colors: Colors, database_metadata: DatabaseMetadataClass, finished: str, made_with: List[str], page_background: str, private: bool, started: str, tags: List[str], thumbnail: str, title_style: str, wip: bool) -> None:
        self.additional_metadata = additional_metadata
        self.aliases = aliases
        self.colors = colors
        self.database_metadata = database_metadata
        self.finished = finished
        self.made_with = made_with
        self.page_background = page_background
        self.private = private
        self.started = started
        self.tags = tags
        self.thumbnail = thumbnail
        self.title_style = title_style
        self.wip = wip

    @staticmethod
    def from_dict(obj: Any) -> 'Metadata':
        assert isinstance(obj, dict)
        additional_metadata = from_dict(lambda x: x, obj.get("additionalMetadata"))
        aliases = from_list(from_str, obj.get("aliases"))
        colors = Colors.from_dict(obj.get("colors"))
        database_metadata = DatabaseMetadataClass.from_dict(obj.get("databaseMetadata"))
        finished = from_str(obj.get("finished"))
        made_with = from_list(from_str, obj.get("madeWith"))
        page_background = from_str(obj.get("pageBackground"))
        private = from_bool(obj.get("private"))
        started = from_str(obj.get("started"))
        tags = from_list(from_str, obj.get("tags"))
        thumbnail = from_str(obj.get("thumbnail"))
        title_style = from_str(obj.get("titleStyle"))
        wip = from_bool(obj.get("wip"))
        return Metadata(additional_metadata, aliases, colors, database_metadata, finished, made_with, page_background, private, started, tags, thumbnail, title_style, wip)

    def to_dict(self) -> dict:
        result: dict = {}
        result["additionalMetadata"] = from_dict(lambda x: x, self.additional_metadata)
        result["aliases"] = from_list(from_str, self.aliases)
        result["colors"] = to_class(Colors, self.colors)
        result["databaseMetadata"] = to_class(DatabaseMetadataClass, self.database_metadata)
        result["finished"] = from_str(self.finished)
        result["madeWith"] = from_list(from_str, self.made_with)
        result["pageBackground"] = from_str(self.page_background)
        result["private"] = from_bool(self.private)
        result["started"] = from_str(self.started)
        result["tags"] = from_list(from_str, self.tags)
        result["thumbnail"] = from_str(self.thumbnail)
        result["titleStyle"] = from_str(self.title_style)
        result["wip"] = from_bool(self.wip)
        return result


class DatabaseValue:
    built_at: str
    content: Dict[str, ContentValue]
    description_hash: str
    id: str
    metadata: Metadata
    partial: bool

    def __init__(self, built_at: str, content: Dict[str, ContentValue], description_hash: str, id: str, metadata: Metadata, partial: bool) -> None:
        self.built_at = built_at
        self.content = content
        self.description_hash = description_hash
        self.id = id
        self.metadata = metadata
        self.partial = partial

    @staticmethod
    def from_dict(obj: Any) -> 'DatabaseValue':
        assert isinstance(obj, dict)
        built_at = from_str(obj.get("builtAt"))
        content = from_dict(ContentValue.from_dict, obj.get("content"))
        description_hash = from_str(obj.get("descriptionHash"))
        id = from_str(obj.get("id"))
        metadata = Metadata.from_dict(obj.get("metadata"))
        partial = from_bool(obj.get("Partial"))
        return DatabaseValue(built_at, content, description_hash, id, metadata, partial)

    def to_dict(self) -> dict:
        result: dict = {}
        result["builtAt"] = from_str(self.built_at)
        result["content"] = from_dict(lambda x: to_class(ContentValue, x), self.content)
        result["descriptionHash"] = from_str(self.description_hash)
        result["id"] = from_str(self.id)
        result["metadata"] = to_class(Metadata, self.metadata)
        result["Partial"] = from_bool(self.partial)
        return result


def database_from_dict(s: Any) -> Dict[str, DatabaseValue]:
    return from_dict(DatabaseValue.from_dict, s)


def database_to_dict(x: Dict[str, DatabaseValue]) -> Any:
    return from_dict(lambda x: to_class(DatabaseValue, x), x)



from typing import List, Any, Dict, Optional, TypeVar, Callable, Type, cast


T = TypeVar("T")


def from_list(f: Callable[[Any], T], x: Any) -> List[T]:
    assert isinstance(x, list)
    return [f(y) for y in x]


def from_str(x: Any) -> str:
    assert isinstance(x, str)
    return x


def from_bool(x: Any) -> bool:
    assert isinstance(x, bool)
    return x


def from_int(x: Any) -> int:
    assert isinstance(x, int) and not isinstance(x, bool)
    return x


def from_dict(f: Callable[[Any], T], x: Any) -> Dict[str, T]:
    assert isinstance(x, dict)
    return { k: f(v) for (k, v) in x.items() }


def from_none(x: Any) -> Any:
    assert x is None
    return x


def from_union(fs, x):
    for f in fs:
        try:
            return f(x)
        except:
            pass
    assert False


def to_class(c: Type[T], x: Any) -> dict:
    assert isinstance(x, c)
    return cast(Any, x).to_dict()


class ExtractColors:
    default_files: List[str]
    enabled: bool
    extract: List[str]

    def __init__(self, default_files: List[str], enabled: bool, extract: List[str]) -> None:
        self.default_files = default_files
        self.enabled = enabled
        self.extract = extract

    @staticmethod
    def from_dict(obj: Any) -> 'ExtractColors':
        assert isinstance(obj, dict)
        default_files = from_list(from_str, obj.get("default files"))
        enabled = from_bool(obj.get("enabled"))
        extract = from_list(from_str, obj.get("extract"))
        return ExtractColors(default_files, enabled, extract)

    def to_dict(self) -> dict:
        result: dict = {}
        result["default files"] = from_list(from_str, self.default_files)
        result["enabled"] = from_bool(self.enabled)
        result["extract"] = from_list(from_str, self.extract)
        return result


class MakeGifs:
    enabled: bool
    file_name_template: str

    def __init__(self, enabled: bool, file_name_template: str) -> None:
        self.enabled = enabled
        self.file_name_template = file_name_template

    @staticmethod
    def from_dict(obj: Any) -> 'MakeGifs':
        assert isinstance(obj, dict)
        enabled = from_bool(obj.get("enabled"))
        file_name_template = from_str(obj.get("file name template"))
        return MakeGifs(enabled, file_name_template)

    def to_dict(self) -> dict:
        result: dict = {}
        result["enabled"] = from_bool(self.enabled)
        result["file name template"] = from_str(self.file_name_template)
        return result


class MakeThumbnails:
    enabled: bool
    file_name_template: str
    input_file: str
    sizes: List[int]

    def __init__(self, enabled: bool, file_name_template: str, input_file: str, sizes: List[int]) -> None:
        self.enabled = enabled
        self.file_name_template = file_name_template
        self.input_file = input_file
        self.sizes = sizes

    @staticmethod
    def from_dict(obj: Any) -> 'MakeThumbnails':
        assert isinstance(obj, dict)
        enabled = from_bool(obj.get("enabled"))
        file_name_template = from_str(obj.get("file name template"))
        input_file = from_str(obj.get("input file"))
        sizes = from_list(from_int, obj.get("sizes"))
        return MakeThumbnails(enabled, file_name_template, input_file, sizes)

    def to_dict(self) -> dict:
        result: dict = {}
        result["enabled"] = from_bool(self.enabled)
        result["file name template"] = from_str(self.file_name_template)
        result["input file"] = from_str(self.input_file)
        result["sizes"] = from_list(from_int, self.sizes)
        return result


class Media:
    at: str

    def __init__(self, at: str) -> None:
        self.at = at

    @staticmethod
    def from_dict(obj: Any) -> 'Media':
        assert isinstance(obj, dict)
        at = from_str(obj.get("at"))
        return Media(at)

    def to_dict(self) -> dict:
        result: dict = {}
        result["at"] = from_str(self.at)
        return result


class Tags:
    repository: str

    def __init__(self, repository: str) -> None:
        self.repository = repository

    @staticmethod
    def from_dict(obj: Any) -> 'Tags':
        assert isinstance(obj, dict)
        repository = from_str(obj.get("repository"))
        return Tags(repository)

    def to_dict(self) -> dict:
        result: dict = {}
        result["repository"] = from_str(self.repository)
        return result


class Technologies:
    repository: str

    def __init__(self, repository: str) -> None:
        self.repository = repository

    @staticmethod
    def from_dict(obj: Any) -> 'Technologies':
        assert isinstance(obj, dict)
        repository = from_str(obj.get("repository"))
        return Technologies(repository)

    def to_dict(self) -> dict:
        result: dict = {}
        result["repository"] = from_str(self.repository)
        return result


class Configuration:
    build_metadata_file: str
    exporters: Optional[Dict[str, Dict[str, Any]]]
    extract_colors: ExtractColors
    make_gifs: MakeGifs
    make_thumbnails: MakeThumbnails
    media: Media
    projects_at: str
    scattered_mode_folder: str
    tags: Tags
    technologies: Technologies

    def __init__(self, build_metadata_file: str, exporters: Optional[Dict[str, Dict[str, Any]]], extract_colors: ExtractColors, make_gifs: MakeGifs, make_thumbnails: MakeThumbnails, media: Media, projects_at: str, scattered_mode_folder: str, tags: Tags, technologies: Technologies) -> None:
        self.build_metadata_file = build_metadata_file
        self.exporters = exporters
        self.extract_colors = extract_colors
        self.make_gifs = make_gifs
        self.make_thumbnails = make_thumbnails
        self.media = media
        self.projects_at = projects_at
        self.scattered_mode_folder = scattered_mode_folder
        self.tags = tags
        self.technologies = technologies

    @staticmethod
    def from_dict(obj: Any) -> 'Configuration':
        assert isinstance(obj, dict)
        build_metadata_file = from_str(obj.get("build metadata file"))
        exporters = from_union([lambda x: from_dict(lambda x: from_dict(lambda x: x, x), x), from_none], obj.get("exporters"))
        extract_colors = ExtractColors.from_dict(obj.get("extract colors"))
        make_gifs = MakeGifs.from_dict(obj.get("make gifs"))
        make_thumbnails = MakeThumbnails.from_dict(obj.get("make thumbnails"))
        media = Media.from_dict(obj.get("media"))
        projects_at = from_str(obj.get("projects at"))
        scattered_mode_folder = from_str(obj.get("scattered mode folder"))
        tags = Tags.from_dict(obj.get("tags"))
        technologies = Technologies.from_dict(obj.get("technologies"))
        return Configuration(build_metadata_file, exporters, extract_colors, make_gifs, make_thumbnails, media, projects_at, scattered_mode_folder, tags, technologies)

    def to_dict(self) -> dict:
        result: dict = {}
        result["build metadata file"] = from_str(self.build_metadata_file)
        if self.exporters is not None:
            result["exporters"] = from_union([lambda x: from_dict(lambda x: from_dict(lambda x: x, x), x), from_none], self.exporters)
        result["extract colors"] = to_class(ExtractColors, self.extract_colors)
        result["make gifs"] = to_class(MakeGifs, self.make_gifs)
        result["make thumbnails"] = to_class(MakeThumbnails, self.make_thumbnails)
        result["media"] = to_class(Media, self.media)
        result["projects at"] = from_str(self.projects_at)
        result["scattered mode folder"] = from_str(self.scattered_mode_folder)
        result["tags"] = to_class(Tags, self.tags)
        result["technologies"] = to_class(Technologies, self.technologies)
        return result


def configuration_from_dict(s: Any) -> Configuration:
    return Configuration.from_dict(s)


def configuration_to_dict(x: Configuration) -> Any:
    return to_class(Configuration, x)
